/*

Monophonic Gap Enforcer
Robert Hart (Bavi_H) (rnhart.net)
2022-10-13

This is a script for MIDI-OX (www.midiox.com), a Windows MIDI monitoring
and mapping tool.

When you open this script in Windows, the script runs in Windows Script
Host. The script opens MIDI-OX (if it's not already open), intercepts
the MIDI messages coming in MIDI-OX's input port, and then sends desired
messages to MIDI-OX's output port. The script keeps track of which pitch
is currently sounding. If another note begins while a pitch is already
sounding, the script will end the first note, pause for a small amount
of time, then begin the new note.


*/

var DELAY = 20;  // milliseconds

var NOTE_OFF = 0x80;
var NOTE_ON  = 0x90;


var mox = WScript.CreateObject("Midiox.MoxScript.1", "On_");


var sounding_channel_bits = -1;
var sounding_pitch = -1;

function On_MidiInput(timestamp, port, status, data1, data2)
{
  var status_type = status & 0xF0;
  var channel_bits = status & 0x0F;

  if( (status_type == NOTE_ON) && (data2 > 0) )  // note begin
  {
    if(sounding_pitch == -1)  // no pitch currently sounding
    {
      mox.OutputMidiMsg(-1, status, data1, data2);
      sounding_channel_bits = channel_bits;
      sounding_pitch = data1;
    }
    else  // pitch already sounding
    {
      mox.OutputMidiMsg(-1, NOTE_OFF | sounding_channel_bits, sounding_pitch, 0);
      WScript.sleep(DELAY);
      mox.OutputMidiMsg(-1, status, data1, data2);
      sounding_channel_bits = channel_bits;
      sounding_pitch = data1;
    }
  }
  else if( (status_type == NOTE_OFF) || ((status_type == NOTE_ON) && (data2 == 0)) )  // note end
  {
    if( (channel_bits == sounding_channel_bits) && (data1 == sounding_pitch) )  // same as sounding pitch and channel
    {
      mox.OutputMidiMsg(-1, status, data1, data2);
      sounding_channel_bits = -1;
      sounding_pitch = -1;
    }
    else  // different from sounding pitch or channel
    {
    }
  }
  else  // all other messages
  {
    mox.OutputMidiMsg(-1, status, data1, data2);
  }

}


function On_SysExInput(sysex)
{
  mox.SendSysExString(sysex)
}


mox.DivertMidiInput = 1;
mox.FireMidiInput = 1;  // begin callbacks

WScript.Echo("Leave this message open for\r\nMIDI stream conversion.\r\n\r\nTo end, press OK.");

mox.FireMidiInput = 0;  // end callbacks

